#!/bin/bash
set -vx 
#to calculate the number of days over/under the climatological monthly 90th/10th percentil and 10th percentil
#chloe prodhomme 27/02/2014


exptype="ecearth2.3" # "ENSEMBLES" #ecearth2.3
explst=b0cm #"cmcc" #b0cm
varlst=tas #tasmax #for ENSEMBLES : tasmax/tasmin

til=90
ENSEMBLE=10
NLT=4
y1=1981
y2=1990

comp[10]="le"
comp[90]="ge"

monlst=(05) #start month
calcdaystat=1 #0 to avoid to calculate dayly stat if done by monthly_pctl.sh

for exp in $explst
do
    #rm -rf /scratch/$(whoami)/$exp
    mkdir /scratch/$(whoami)/$exp
    work=/scratch/$(whoami)/work-nbdays/${exp}
    #rm -rf $work
    mkdir /scratch/$(whoami)/work-nbdays/
    mkdir $work
    for var in $varlst 
    do
	for monstrt in $monlst
	do

            #generate the header 
	    case $exptype in
		"ecearth2.3") 
		    data=/cfunas/exp/ecearth/${exp}/daily_grb/${var}_6hourly
		    dataout=/cfunas/exp/ecearth/${exp}/monthly_statistics/
		    mkdir $dataout
		    ncks -O -h -x -v  $var,latitude,longitude,level_$var ${data/daily_grb/monthly_mean}/${var}_${y1}${monstrt}01.nc ${work}/header.nc ;;
		ENSEMBLES)
		    data=/cfunas/exp/ENSEMBLES/seasonal/${exp}/daily/${var}/
		    dataout=/cfunas/exp/ENSEMBLES/seasonal/${exp}/monthly_statistics/
		    mkdir $dataout
		    ncks -x -O -h -v  $var,latitude,longitude $data/${var}_${y1}${monstrt}01.nc ${work}/header-aux.nc 
		    ncks -d time,1,$NLT ${work}/header-aux.nc  ${work}/header.nc
		    #if you want to have a proper time and time_bnd value modify : ${work}/header.nc
		    ;;		
	    esac
	    
	    for year in `seq $y1 $y2`
	    do
		


		case $exptype in
		    "ecearth2.3")
			if [[ $var == "prlr" ]]
			then
			    varauxlst="lsp cp"
			    var2=prlr
			    freq=24
			    oldname="LSP"
			    daystat=sum
			fi
			if [[ $var == "tas" ]]
			then
			    if [[ $til == "10" ]]
			    then
				daystat=min
			    fi
			    if [[ $til == "90" ]]
			    then
				daystat=max
			    fi
			    varauxlst="tas"
			    var2=tas$daystat
			    oldname="T2M"
			    freq=6
			fi			   
			dirdaystat=/scratch/$(whoami)/${exp}/${var2}
#calculate daily stat in the "ecearth2.3" case
			if [[ $calcdaystat == "1" ]]
			then

			    for varaux in $varauxlst
			    do			    
				data=/cfunas/exp/ecearth/${exp}/daily_grb/${varaux}_6hourly
				file=${varaux}_${year}${monstrt}01
				DATE=${year}${monstrt}01
		            #number of timestep
				step=$(( (($(date -u --date=${DATE}+${NLT}month +%s) - $(date -u --date=${DATE} +%s) ))/(60*60*$freq)))
				ls $work/${file}.nc || cdo -R -r -t ecmwf -f nc copy $data/$file.grb $work/${file}.nc
                            #loop over the members
				for r in `seq 1 $ENSEMBLE`
				do
				    ncks -O -d time,$(($r-1)),,$ENSEMBLE $work/${file}.nc $work/${file}_r${r}.nc 
				    ncks -O -d time,0,$(($step-1)) $work/${file}_r${r}.nc $work/${file}_r${r}.nc2
				    mv $work/${file}_r${r}.nc2 $work/${file}_r${r}.nc
                                #calculate min, max sum or mean according to the variable
				    cdo -r day${daystat} $work/${file}_r${r}.nc $work/day${daystat}_${file}_r${r}.nc
				done #r
		            #concatenate file
				file=${varaux}_${year}${monstrt}01.nc
				ncecat -h -O $work/day${daystat}_${varaux}_${year}${monstrt}01_r*.nc $work/aux_$file
				ncpdq -O -h -a time,record $work/aux_${file} $work/aux_${file}
				mv $work/aux_${file} $work/$file
				
			    done
			    if [[ $var == "prlr" ]]
			    then
				cdo -r add $work/lsp_${year}${monstrt}01.nc $work/cp_${year}${monstrt}01.nc $work/prlr_${year}${monstrt}01.nc	
		    		file=prlr_${year}${monstrt}01.nc
			    fi
			    ncrename -O -v $oldname,$var2 $work/$file
                        #copy daily stat for calculate the climatology
			    mkdir /scratch/$(whoami)/${exp}/
			    mkdir $dirdaystat
			    cp $work/$file $dirdaystat/${var2}_${year}${monstrt}01.nc
			    file=${var2}_${year}${monstrt}01.nc
			fi
			;;

		    

		    #in ensemble case just the set the time axis of the daily file
		    ENSEMBLES)
			
			var2=$var	
			dirdaystat=/scratch/$(whoami)/${exp}/${var2}
			mkdir /scratch/$(whoami)/${exp}/
			mkdir $dirdaystat
			if [[ $calcdaystat == "1" ]]
			then
			    file=${var}_${year}${monstrt}01.nc
			    cp $data/$file $work/$file
			    cdo -r settaxis,$year-${monstrt}-01,12:00,1day $work/$file $work/taxis_$file
			    cp  $work/taxis_$file $dirdaystat/$file
			    file=taxis_$file
			fi
			;;
		esac
	    done  
	    
	    rm -rf $work/*
	    yaux=1000 #start from 1000 to always have 4 digits
	    for year in `seq $y1 $y2`
	    do		
		file=${var2}_${year}${monstrt}01.nc
		    #loop over member
		    #in order to make believe to cdo that each member represent a different year
		for r in `seq 1 $ENSEMBLE`
		do
			#select one member
		    fileaux=${var2}_${yaux}${monstrt}01.nc
		    cdo sellevel,$r $dirdaystat/$file $work/${fileaux}
		#remove level dim
		     ncwa -O -a lev  $work/${fileaux}  $work/nolev_${fileaux}
		    fileaux=nolev_${fileaux}
		    cdo -r settaxis,$yaux-${monstrt}-01,12:00,1day $work/$fileaux $work/taxis_$fileaux
		    yaux=$(( yaux + 1 ))
		done
	    done
	    cdo mergetime $work/taxis_nolev_${var2}_????${monstrt}01.nc $work/cat_taxis_${var2}_1000-${yaux}_${monstrt}01.nc
	    file_out=cat_taxis_${var2}_1000-${yaux}_${monstrt}01.nc

	     #calculate the climatological percentil
	    cdo -r ymonmin $work/$file_out $work/ymonmin_${file_out}
	    cdo -r ymonmax $work/$file_out $work/ymonmax_${file_out}
	    climtil=ymonq${til}_${file_out}
	    cdo -r ymonpctl,$til $work/$file_out $work/ymonmin_${file_out} $work/ymonmax_${file_out} $work/$climtil 

	    for year in `seq $y1 $y2`
	    do
	    	#transform monthly file in daily file
		#need to be put in the loop for february
		for imon in  `seq 0 $((NLT-1))`
		do
		    climtil_mon=${climtil%.nc}_m$imon.nc
		    cdo selmon,$((monstrt+imon)) $work/$climtil $work/$climtil_mon
		# dans la boucle pour 29 feb
		    DATE=${year}${monstrt}01
		    nbday=$(( (($(date -u --date=${DATE}+$((imon+1))month +%s) - $(date -u --date=${DATE}+$((imon))month +%s) ))/(60*60*24)))
		    echo $nbday
		    for iday in `seq 1 $nbday`
		    do
			cp $work/$climtil_mon $work/toto$iday.nc				
		    done
		    climtil_day=daily_$climtil_mon
		    ncrcat -O $work/toto*.nc $work/daily_$climtil_mon
		    rm -rf $work/toto*.nc
		    
		done
		climtil_comp=daily_$climtil
		ncrcat -O $work/daily_ymonq${til}_${file_out%.nc}_m*.nc  $work/$climtil_comp
	    #recreate artifically 10ens member
		for iens in `seq 1 $ENSEMBLE`
		do
		    cp $work/$climtil_comp $work/toto$iens.nc				
		done
		
		ncecat -O $work/toto*.nc $work/ens_$climtil_comp
		climtil_comp=ens_$climtil_comp
		rm -rf $work/toto*
		ncpdq -O -h -a time,record $work/${climtil_comp} $work/${climtil_comp}2

		mv $work/${climtil_comp}2 $work/${climtil_comp}
	    #ncwa -O  -h -a record $work/${climtil_comp}2 $work/${climtil_comp}3
		
		
#comparison between the daily min/max/sum and the comparison daily climtil just created
		file=$dirdaystat/${var2}_${year}${monstrt}01.nc
		cdo -r ${comp[til]} $file $work/$climtil_comp $work/${comp[til]}q${til}_${var2}_${year}${monstrt}01.nc
		
			#sum to known the number of days
		cdo -r monsum  $work/${comp[til]}q${til}_${var2}_${year}${monstrt}01.nc $work/nbdaysq${til}_${var2}_${year}${monstrt}01.nc
		
		file=nbdaysq${til}_${var2}_${year}${monstrt}01.nc

		ncrename -O -h -v ${var2},${var2}_nbdaysq${til}_${y1}-${y2} $work/${file} $work/${file}1		
		ncrename -O -h -d lev,ensemble $work/${file}1 $work/${file}2
		cp $work/${file}2 $work/${file}6
		if [[ $exptype == "ecearth2.3" ]]
		then
		    ncrename -O -h -d lon,longitude $work/${file}2 $work/${file}3
		    ncrename -O -h -d lat,latitude $work/${file}3 $work/${file}4
		    ncrename -O -h -v lon,longitude $work/${file}4 $work/${file}5
		    ncrename -O -h -v lat,latitude $work/${file}5 $work/${file}6
		fi
		ncrename -h -v time,leadtime $work/${file}6
		#ncatted -h -O -a long_name,leadtime,c,c,"time elapsed since the start of the forecast" $work/${file}6

		ncks -A $work/header.nc  $work/${file}6
		ncecat -O -h $work/${file}6 $work/${file}7
		ncpdq -O -h -a time,record $work/${file}7 $work/${file}8
		ncwa -O  -h -a record $work/${file}8 $work/${file}9
		ncks -x -O -h -v lev $work/${file}9 $work/${file}10

		mkdir ${dataout}/${var2}_nbdaysq${til}_${y1}-${y2}

		mv $work/${file}10 ${dataout}/${var2}_nbdaysq${til}_${y1}-${y2}/${var2}_nbdaysq${til}_${y1}-${y2}_${year}${monstrt}01.nc
		chmod 770 ${dataout}/${var2}_nbdaysq${til}_${y1}-${y2}/${var2}_nbdaysq${til}_${y1}-${y2}_${year}${monstrt}01.nc
	    done #yearlst
	    #rm -rf /scratch/$(whoami)/$exp/*
	    #rm -rf $work/*
	done #monlst
    done #varlst
done #explst